import 'package:flutter/material.dart';
import 'package:timelines/timelines.dart';
import 'package:transit_navigation_view/transit_navigation_view.dart';
import 'package:transit_navigation_view/utility.dart';

class DetailViewContent extends StatelessWidget {
  final TransitData transitData;
  late final TextStyle headerTextStyle;
  late final TextStyle transportTextStyle;
  late final TextStyle feeTextStyle;
  late final TextStyle stopTextStyle;
  late final TextStyle stopTimeTextStyle;

  DetailViewContent({
    Key? key,
    required this.transitData,
  }) : super(key: key) {
    headerTextStyle = const TextStyle(
      fontSize: 16,
      fontWeight: FontWeight.bold,
    );

    stopTextStyle = const TextStyle(
      fontSize: 18,
      fontWeight: FontWeight.bold,
    );

    stopTimeTextStyle = const TextStyle(
      fontSize: 16,
      fontWeight: FontWeight.bold,
    );

    transportTextStyle = const TextStyle(
      fontSize: 16,
      // fontWeight: FontWeight.w600,
    );

    feeTextStyle = const TextStyle(
      fontSize: 14,
    );
  }

  @override
  Widget build(context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        _buildDetailTabHeader(context),
        Expanded(
          child: _buildDetailTabTimeTable(),
        ),
      ],
    );
  }

  Widget _buildDetailTabHeader(BuildContext context) {
    final showWalkTime = transitData.walkTime != null;
    return Container(
      color: Theme.of(context).colorScheme.background,
      padding: const EdgeInsets.all(10),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.start,
            mainAxisSize: MainAxisSize.min,
            children: [
              Text(
                "${transitData.dptTime} ▶ ${transitData.arvTime}",
                style: headerTextStyle,
              ),
              // const SizedBox(width: 30),
              const Spacer(),
              showWalkTime
                  ? Text(
                      "${transitData.totalTime} （徒歩：${transitData.walkTime}）",
                      style: headerTextStyle,
                    )
                  : Text(
                      "所要時間：${transitData.totalTime}",
                      style: headerTextStyle,
                    ),
              const Spacer(),
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.start,
            mainAxisSize: MainAxisSize.min,
            children: [
              Text(
                "運賃合計： " + intToPriceString(transitData.totalFee),
                style: headerTextStyle,
              ),
              const Spacer(),
              Text(
                "乗り換え： " + (transitData.numOfTransit).toString() + "回",
                style: headerTextStyle,
              ),
              const Spacer(),
            ],
          ),
        ],
      ),
    );
  }

  _buildDetailTabTimeTable() {
    if (transitData.stationDataList.isEmpty ||
        transitData.passDataList.isEmpty) {
      return Container();
    }
    final length =
        transitData.stationDataList.length + transitData.passDataList.length;

    return Align(
      alignment: Alignment.topLeft,
      child: Timeline.tileBuilder(
        builder: TimelineTileBuilder.connected(
          connectionDirection: ConnectionDirection.before,
          itemCount: length,
          addRepaintBoundaries: false,
          contentsAlign: ContentsAlign.basic,
          oppositeContentsBuilder: (context, index) =>
              _buildOppositeNode(index),
          contentsBuilder: (context, index) => _buildNode(index),
          indicatorBuilder: (context, index) => _buildIndicator(index),
          connectorBuilder: _buildConnector,
          nodePositionBuilder: (context, index) => 0.2,
        ),
      ),
    );
  }

  Widget _buildNode(int index) {
    final isEven = index % 2 == 0;
    if (isEven) {
      return _buildStopNode(transitData.stationDataList[index ~/ 2]);
    } else {
      return _buildTransportNode(transitData.passDataList[index ~/ 2]);
    }
  }

  Widget _buildStopNode(StationData stationData) {
    return Container(
      margin: const EdgeInsets.fromLTRB(0, 20.0, 20.0, 20.0),
      padding: const EdgeInsets.all(10),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10),
        color: Colors.grey[200],
      ),
      child: Row(
        children: [
          Flexible(
            child: Text(
              stationData.stationString,
              style: stopTextStyle,
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildTransportNode(PassData passData) {
    return Container(
      margin: const EdgeInsets.fromLTRB(10, 10, 30, 10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Flexible(
            child: Text(
              passData.passString,
              style: transportTextStyle,
            ),
          ),
          const SizedBox(width: 10),
          Text(
            passData.feeString,
            style: feeTextStyle,
          ),
        ],
      ),
    );
  }

  Widget _buildOppositeNode(int index) {
    final isEven = index % 2 == 0;
    if (isEven) {
      return _buildStopTimeNode(transitData.stationDataList[index ~/ 2]);
    } else {
      return _buildTransportTimeNode(transitData.passDataList[index ~/ 2]);
    }
  }

  Widget _buildStopTimeNode(StationData stationData) {
    return Container(
      margin: const EdgeInsets.all(10),
      child: Text(
        stationData.timeString,
        style: stopTimeTextStyle,
      ),
    );
  }

  Widget _buildTransportTimeNode(PassData passData) {
    return Container(
      margin: const EdgeInsets.all(10),
      child: Text(
        passData.timeString,
        style: transportTextStyle,
      ),
    );
  }

  Widget _buildIndicator(int index) {
    final isEven = index % 2 == 0;
    if (isEven) {
      return _stopIndicator();
    } else {
      final passData = transitData.passDataList[index ~/ 2];
      final stationData = transitData.stationDataList[index ~/ 2];
      if (passData.passType == PassType.walk) {
        return _walkIndicator();
      } else if (passData.passType == PassType.subway) {
        return _subwayIndicator();
      } else {
        return _busIndicator();
      }
    }
  }

  Widget _indicatorWrapper(Widget child) {
    return SizedBox(
      // height: 20,
      width: 30,
      child: Center(
        child: Container(
          // margin: const EdgeInsets.all(10),
          child: child,
        ),
      ),
    );
  }

  Widget _stopIndicator() {
    return _indicatorWrapper(
      const OutlinedDotIndicator(
        size: 15.0,
        color: Colors.grey,
        borderWidth: 3.0,
      ),
    );
  }

  Widget _busIndicator() {
    return _iconIndicator(Icons.directions_bus, Colors.redAccent);
  }

  Widget _subwayIndicator() {
    return _iconIndicator(Icons.directions_transit, Colors.amber);
  }

  Widget _walkIndicator() {
    return _iconIndicator(Icons.directions_walk, Colors.lightBlue);
  }

  Widget _iconIndicator(IconData icon, Color color) {
    return _indicatorWrapper(
      DotIndicator(
        color: color,
        size: 25.0,
        child: Icon(
          icon,
          color: Colors.white,
          size: 20.0,
        ),
      ),
    );
  }

  Widget? _buildConnector(
    BuildContext context,
    int index,
    ConnectorType connectorType,
  ) {
    final isEven = index % 2 == 0;
    if (isEven) {
      index -= 1;
    }

    final passData = transitData.passDataList[index ~/ 2];
    final stationData = transitData.stationDataList[index ~/ 2];
    if (passData.passType == PassType.walk) {
      return const DashedLineConnector(color: Colors.lightBlue);
    } else if (passData.passType == PassType.subway) {
      return const SolidLineConnector(color: Colors.amber);
    } else {
      return const SolidLineConnector(color: Colors.redAccent);
    }
  }
}
